#!/usr/local/bin/perl

# *********************************************************************** #
# *  Calendar Program v3.0             05-Feb-2004                      * #
# *                                                                     * #
# *  This calendar is updated from the original version 2.1 produced    * #
# *  by Scott M. Crevier. The main reason for this update was the       * #
# *  ability to present the calendar in a small version, with only      * #
# *  dates and links to be used in conjuction with the CMS Coranto.     * #
# *  Coranto can be found at http://coranto.gweilo.org                  * #
# *  The same license and liability applies as before!!!                * #
# *                                                                     * #
# *  henrik@parahead.com    -    Henrik Ahln    -    www.parahead.com  * #
# *********************************************************************** #

# *********************************************************************** #
# *									* #
# *                    Copyright (c) 1997-2001 smcNet			* #
# *                         All Rights Reserved				* #
# *									* #
# *  This program product material is the property of Scott Crevier,	* #
# *  dba/smcNet. This program may be used and modified by anyone free   * #
# *  of charge, as long as this copyright notice remains in tact. By	* #
# *  using this	code, you agree to indemnify Scott Crevier and smcNet	* #
# *  from any liability	that might arise from its use.			* #
# *									* #
# *  webmaster@smcnet.com  -    Scott M. Crevier     -  www.smcnet.com  * #
# *********************************************************************** #
# *  Calendar Program v2.1				02-Jun-2001	* #
# *									* #
# *  This is a web based calendar that is driven by a simple text	* #
# *  file containing calendar events.					* #
# *									* #
# *  DO NOT MAKE ANY CHANGES IN THIS FILE UNLESS YOU KNOW WHAT		* #
# *  YOU'RE DOING! All customizations can be made in the 'config.pl'	* #
# *  file, which is included in this package.				* #
# *									* #
# *********************************************************************** #

%configfiles = (
					'default' => 'default_config.pl',
					'birthdays' => 'birthdays.pl'
               );

require "timelocal.pl";

&initializeVars;
&checkForErrors;
&readCalendarData;

if ($option eq 'list') {
	&printList;
}
elsif($option eq 'small') {
	&printSmallTableTop;
	&printSmallPrevNext;
	&printSmallWeekDays;
	&printSmallCalendar;
	&printSmallTableBottom;		
} else {
	&printPageTop;
	&printTableTop;
	&printCalendar;
	&printTableBottom;
}

exit(0);


# *********************************************************************** #
# *  Check for errors.							* #
# *********************************************************************** #
sub checkForErrors {

	my $libfile = $configfiles{$config};	
	
	if (-r $libfile) {
		require $libfile;
	} else {
		$msg	= "Cannot read/find library file '<font color=blue>$libfile</font>'\n";
		&gracefulError($msg);
	}

	unless (-r $data) {
		$msg	= "Cannot read data file '<font color=blue>$data</font>'\n"
			. "<p>\n"
			. "You may want to try one of the following:\n"
			. "<ul>\n"
			. "<li>Move your events file so its filename and location match what's shown above.\n"
			. "<li>Change the calendar script so that it's looking for the events file somewhere else.\n"
			. "<li>Change the permissions on your events file. A good UNIX command would be "
			. "'chmod 604 <i>filename</i>'."
			. "</ul>\n";
		&gracefulError($msg);
	}
}



# *********************************************************************** #
# *  Initialize all the date-related variables.				* #
# *********************************************************************** #
sub initializeVars {
	$thisurl	= $ENV{REQUEST_URI};
	$thisurl	=~ s/\?.*//;
	$caltitlejs	= $caltitle;
	$caltitlejs	=~ s/\'/\\'/g;

	# *************************************************************** #
	&getOptions(*FORM);
	$option	= $FORM{o};
	$style	= $FORM{s};
	$config = defined($FORM{c}) ? $FORM{c}: 'default';
	
	# *************************************************************** #
	# *  Get today's date.						* #
	# *************************************************************** #
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

	$today_yyyy	= $year + 1900;
	$today_yy   = substr($today_yyyy,2);
	$today_mm	= sprintf("%02d", $mon + 1);
	$today_dd	= sprintf("%02d", $mday);
	$today		= $today_yyyy . $today_mm . $today_dd;


	# *************************************************************** #
	# *  Get the month and year being requested.			* #
	# *************************************************************** #
	if( !defined($FORM{'yyyy'}) && !defined($FORM{'mm'}) ) {
		$yyyy	= $today_yyyy;
		$mm	= $today_mm;
	} else {
		$yyyy	= $FORM{'yyyy'};
		$mm	= $FORM{'mm'};
	}

	$yyyy	= ($year + 1900) if ($yyyy  eq '');
	$yy   = substr($yyyy,2);
	$mm	= sprintf("%02d", $mon + 1) if ($mm eq '');

	# *************************************************************** #
	# *  Set $year and $mon to values that the 'localtime' function	* #
	# *  will understand. These values can be verified in the ctime	* #
	# *  or localtime man pages.					* #
	# *************************************************************** #
	$year = ($yyyy - 1900);
	$mon  = ($mm - 1);


	# *************************************************************** #
	# *  Get the month and year for last month and next month.	* #
	# *************************************************************** #
	$prevyyyy	= $yyyy;
	$prevmm		= sprintf("%02d", $mm - 1);
	if ($prevmm < 1) { $prevmm = 12; $prevyyyy--; }

	$nextyyyy	= $yyyy;
	$nextmm		= sprintf("%02d", $mm + 1);
	if ($nextmm > 12) { $nextmm = '01'; $nextyyyy++; }


	# *************************************************************** #
	# *  Get first date of current month.				* #
	# *************************************************************** #
	$mday	= 1;
	$sec	= $min = $hour = $wday = $yday = $isdst = 0;
	$idate	= &timelocal($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($idate);


	# *************************************************************** #
	# *  Get first date of previous month.				* #
	# *************************************************************** #
	$prevmon	= ($mon - 1);
	$prevyear	= $year;
	if ($prevmon < 0) {
		$prevmon = 11;
		$prevyear--;
	}

	$idate	= &timelocal($sec,$min,$hour,$mday,$prevmon,$prevyear,$prevwday,$yday,$isdst);
	($sec,$min,$hour,$mday,$prevmon,$prevyear,$prevwday,$yday,$isdst) = localtime($idate);


	# *************************************************************** #
	# *  Get first date of next month.				* #
	# *************************************************************** #
	$nextmon	= ($mon + 1);
	$nextyear	= $year;
	if ($nextmon == 12) {
		$nextmon = 0;
		$nextyear++;
	}

	$idate	= &timelocal($sec,$min,$hour,$mday,$nextmon,$nextyear,$nextwday,$yday,$isdst);
	($sec,$min,$hour,$mday,$nextmon,$nextyear,$nextwday,$yday,$isdst) = localtime($idate);


	# *************************************************************** #
	# *  Get last date of current month.				* #
	# *************************************************************** #
	$idate = ($idate - 86400);
	($sec,$min,$hour,$mday,$lastmon,$lastyear,$lastwday,$yday,$isdst) = localtime($idate);
}


# *********************************************************************** #
# *  Read in the data file and build the array of calendar data. If	* #
# *  the first character of any line does not begin with a 0, 1 or 2,	* #
# *  then it's not calendar data so we ignore the line.			* #
# *********************************************************************** #
sub readCalendarData {
	my $lcv = 0;

	open(DATA,$data) or die "$! \'$data\'";
	while (<DATA>) {
		next unless (/^[0,1,2]/);
		chomp;
		push(@caldata,"${lcv}${sep}$_");
		$lcv++;
	}
	close(DATA);
}


# *********************************************************************** #
# *  Print a list of events.						* #
# *********************************************************************** #
sub printList {
	my($yyyy,$mm,$dd,$lcv,$days,$dndx,$today,$lastdate,$i,$jsmsgcnt,@msgs);
	my($ndx,$date,$txcolor,$bgcolor,$title,$url,$msg);

	print "Content-type: text/html\n\n";

	$lcv = 0;
	$jsmsgcnt = 0;

	# *************************************************************** #
	# *  Handle annual events.					* #
	# *************************************************************** #
	$today_mmdd	= $today_mm . $today_dd;

	for ($i = 0; $i <= $#caldata; $i++) {
		(@f) = split(/\Q$sep/,$caldata[$i]);

		next unless ($f[1] =~ /^0000/);

		$mm	= substr($f[1],4,2);
		$dd	= substr($f[1],6,2);
		$mmdd	= $mm . $dd;

		$f[1]	= ($mmdd < $today_mmdd) ? ($today_yyyy + 1) . $mmdd : $today_yyyy . $mmdd;

		$caldata[$i] = join($sep,@f);

	}

	@caldata = sort {
		(split(/\Q$sep/,$a))[1] cmp (split(/\Q$sep/,$b))[1]
		or
		(split(/\Q$sep/,$a))[3] cmp (split(/\Q$sep/,$b))[3]
		or
		$a cmp $b;
	} @caldata;


	# *************************************************************** #
	# *  Print events for current month only.			* #
	# *************************************************************** #
	if ($style eq 'current') {
		foreach (@caldata) {
			($ndx,$date,$txcolor,$bgcolor,$title,$url,$msg) = split(/\Q$sep/);

			$yyyy	= substr($date,0,4);
			$mm	= substr($date,4,2);
			$dd	= substr($date,6,2);

			next unless (($yyyy eq $today_yyyy) && ($mm eq $today_mm));

			$dd	-= 0;
			$dndx	= &dayofweek($yyyy,$mm,$dd);
			$date	= $weekdays[$dndx] . ', ' . $months[$mm - 1] . ' ' . $dd;

			if ($msg ne '') {
				$url		= "javascript:msgAlert(${ndx})";
				$msgs[$ndx]	= $title . '\n' . $msg;
				$msgs[$ndx]	=~ s/<br>/\\n/g;
				$msgs[$ndx]	=~ s/'/\\'/g;

				$jsmsgcnt++;
			}

			$title	= "<a href=\"$url\">$title</a>" if ($url ne '');

			print	"<tr>\n",
				"<td class=\"smc\">$date</td>\n",
				"<td class=\"smc\">$title</td>\n",
				"</tr>\n";
			$lcv++;
		}

		unless ($lcv) {
			print	"<tr>\n",
				"<td class=\"smc\" colspan=2 align=center>&#149; No Events This Month &#149;</td>\n",
				"</tr>\n";
		}
	}

	# *************************************************************** #
	# *  Print events for the next x days only.			* #
	# *************************************************************** #
	elsif ($style =~ /^days(\d+)$/) {

		$days		= $1;
		$today		= $today_yyyy . $today_mm . $today_dd;
		$lastdate	= &dateadd($today_yyyy,$today_mm,$today_dd,$days);

		print	"<!-- Printing events from $today to $lastdate -->\n";

		foreach (@caldata) {
			($ndx,$date,$txcolor,$bgcolor,$title,$url,$msg) = split(/\Q$sep/);

			next unless (($date >= $today) && ($date <= $lastdate));

			$yyyy	= substr($date,0,4);
			$mm	= substr($date,4,2);
			$dd	= substr($date,6,2);

			$dd	-= 0;
			$dndx	= &dayofweek($yyyy,$mm,$dd);
			$date	= $weekdays[$dndx] . ', ' . $months[$mm - 1] . ' ' . $dd;

			if ($msg ne '') {
				$url		= "javascript:msgAlert(${ndx})";
				$msgs[$ndx]	= $title . '\n' . $msg;
				$msgs[$ndx]	=~ s/<br>/\\n/g;
				$msgs[$ndx]	=~ s/'/\\'/g;

				$jsmsgcnt++;
			}

			$title	= "<a href=\"$url\">$title</a>" if ($url ne '');

			print	"<tr>\n",
				"<td class=\"smc\">$date</td>\n",
				"<td class=\"smc\">$title</td>\n",
				"</tr>\n";
			$lcv++;
		}
		unless ($lcv) {
			if    ($days == 0)	{ $msg = "No events for today."; }
			elsif ($days == 1)	{ $msg = "No events for today or tomorrow."; }
			else			{ $msg = "No events for the next $days days."; }

			print	"<tr>\n",
				"<td class=\"smc\" colspan=2 align=center>$msg</td>\n",
				"</tr>\n";
		}
	}

	# *************************************************************** #
	return unless ($jsmsgcnt);

	print	"<script type=\"text/javascript\" language=\"javascript\">\n",
		"<!--\n",
		"\tfunction msgAlert (i) {\n",
		"\t\talert('$caltitlejs\\n\\n' + msg[i]);\n",
		"\t}\n",
		"\tmsg = new Array();\n";

	for ($i = 0; $i <= $#msgs; $i++) {
		if ($msgs[$i] ne '') {
			$msgs[$i] =~ s/<br>/\\n/g;
			print	"\tmsg[$i] = '$msgs[$i]';\n";
		}
	}

	print	"// -->\n",
		"</script>\n";

}


# *********************************************************************** #
# *  Print the top of the HTML page.					* #
# *********************************************************************** #
sub printPageTop {
	print "Content-type: text/html\n\n";
	&printFile($header);
}


# *********************************************************************** #
# *  Print the top of the calendar table.				* #
# *********************************************************************** #
sub printTableTop {
	my($prev,$next);

	# *************************************************************** #
	$prev	= "<a href=\"${thisurl}?c=$config&mm=$prevmm&yyyy=$prevyyyy\" "
		. "onMouseOver=\"window.status='$month[$prevmm - 1] $prevyyyy';return true\" "
		. "onMouseOut=\"window.status=''\">"
		. "<img src=\"$arrow_left\" border=0 alt=\"$month[$prevmm - 1] $prevyyyy\"></a>";

	$next	= "<a href=\"${thisurl}?c=$config&mm=$nextmm&yyyy=$nextyyyy\" "
		. "onMouseOver=\"window.status='$month[$nextmm - 1] $nextyyyy';return true\" "
		. "onMouseOut=\"window.status=''\">"
		. "<img src=\"$arrow_right\" border=0 alt=\"$month[$nextmm - 1] $nextyyyy\"></a>";


	# *************************************************************** #
	print	"<center>\n",
		"<table border=${table_border} cellpadding=${cellpadding} cellspacing=${cellspacing}>\n";

	print	"<tr><td colspan=7>\n";
	print	"<table width=\"100%\" cellpadding=0 border=0><tr>\n",
		"<td width=\"8%\" valign=top align=left>$prev</td>\n";

	print	"<form method=\"get\" action=\"$thisurl\">\n" if ($header_style eq 'B');
	print	"<td width=\"36%\" valign=middle align=center>";

	if ($header_style eq 'A') {
		print	"<font size=\"+2\">$month[$mon] $yyyy</font>\n";
	} elsif ($header_style eq 'B') {

		print	"<table border=0 cellpadding=2 cellspacing=0><tr>\n";

		print	"<td><select name=\"mm\" size=1>\n";
		for $n (1..12) {
			$nx	= $n - 1;
			$n	= sprintf("%02d",$n);
			$sel	= ($nx == $mon) ? ' selected' : '';
			print	"<option value=\"$n\"${sel}>$month[$nx]\n";
		}
		print	"</select></td>\n";

		print	"<td><select name=\"yyyy\" size=1>\n";
		for $n (-2..4) {
			$y	= ($today_yyyy + $n);
			$sel	= ($y == $yyyy) ? ' selected' : '';
			print	"<option value=\"$y\"${sel}>$y\n";
		}
		print	"</select></td>\n";

		print   "<input type=\"hidden\" name=\"c\"value=\"$config\">\n";
		print	"<td><input type=\"submit\" value=\"Go\"></td>\n";

		print	"</tr></table>\n";

	} else {
		print	"<tt>UNRECOGNIZED HEADER STYLE: '$header_style'</tt>\n";
	}

	print	"</td>\n";
	print	"</form>\n" if ($header_style eq 'B');

	print	"<td width=\"8%\" valign=top align=right>$next</td>\n",
		"</tr></table>\n";


	print	"</td></tr>\n";

	# *************************************************************** #
	print	"<tr bgcolor=#ffffc0>\n",
		"<th width=${colwidth}>$weekday[0]</th>\n",
		"<th width=${colwidth}>$weekday[1]</th>\n",
		"<th width=${colwidth}>$weekday[2]</th>\n",
		"<th width=${colwidth}>$weekday[3]</th>\n",
		"<th width=${colwidth}>$weekday[4]</th>\n",
		"<th width=${colwidth}>$weekday[5]</th>\n",
		"<th width=${colwidth}>$weekday[6]</th>\n",
		"</tr>\n";
}


# *********************************************************************** #
# *  Print the main calendar data.					* #
# *********************************************************************** #
sub printCalendar {

	$colspanvar	= $wday;
	$begindates	= 0;
	$daynumber	= 0;

	print	"<tr valign=top>\n";

	# *************************************************************** #
	# *  Start the dates loop.					* #
	# *************************************************************** #
	DAY:
	for ($i = 1; $i <= $mday; $i++) {
		$daynumber++;

		# ******************************************************* #
		# *  Handle the beginning of the month.			* #
		# ******************************************************* #
		if ( !($begindates) && ($colspanvar > 0) ) {
			print "<td colspan=\"$colspanvar\"><br></td>\n";
			$begindates = 1;
			$daynumber = $colspanvar;
			$i = 0;
			next DAY;
		}

		# ******************************************************* #
		# *  Generate the date string that will be sent to the	* #
		# *  printEvents subroutine.				* #
		# ******************************************************* #
		$dd = ($i < 10) ? '0' . $i : $i;
		$caldate = $yyyy . $mm . $dd;

		# ******************************************************* #
		# *  Print the HTML code for this date.			* #
		# ******************************************************* #
		&printEvents($caldate);

		# ******************************************************* #
		# *  Handle then end of week.				* #
		# ******************************************************* #
		$mod = ($daynumber % 7);
		if ( ($mod == 0) && ($i < $mday) ) {
			#print "</tr>\n<!--[$daynumber][$i][$dd]-->\n<tr valign=top>\n";
			print "</tr>\n<tr valign=top>\n";
		}
	}

	# *************************************************************** #
	# *  Handle then end of month.					* #
	# *************************************************************** #
	if ( $mod != 0 ) {
		$colspanvar = ( 7 - $mod);
		print "<td colspan=\"$colspanvar\"><br></td>\n";
	}

	print	"</tr>\n\n";

	print	"</table>\n",
		"</center>\n";

	# *************************************************************** #
	print	"<script type=\"text/javascript\" language=\"javascript\">\n",
		"<!--\n",
		"\tfunction msgAlert (i) {\n",
		"\t\talert('$caltitlejs\\n\\n' + msg[i]);\n",
		"\t}\n",
		"\tmsg = new Array();\n";

	for ($i = 0; $i <= $#msgs; $i++) {
		if ($msgs[$i] ne '') {
			$msgs[$i] =~ s/<br>/\\n/g;
			print	"\tmsg[$i] = '$msgs[$i]';\n";
		}
	}

	print	"// -->\n",
		"</script>\n";

}

# *********************************************************************** #
# *  Print the bottom of the calendar table.				* #
# *********************************************************************** #
sub printTableBottom {
	&printFile($footer);
}



# *********************************************************************** #
# *  Subroutine to print the contents of a text file.			* #
# *********************************************************************** #
sub printFile {
	my($filename) = @_;

	if (-r $filename) {
		open(FILE,$filename) or die "$! \'$filename\'";
		while (<FILE>) { print; }
		close(FILE);
	} else {
		print	"\n\n<!-- WARNING: File not found '$filename' -->\n\n";
	}
}


# *********************************************************************** #
# *  Subroutine to find and print calendar data. The date passed is in	* #
# *  the form 'yyyymmdd'.						* #
# *********************************************************************** #
sub printEvents {
	local($caldate) = @_;
	local($date,$size,$category,$color,$string);
	local($yyyy,$mmdd);
	local(@events);

	my($date,$txcolor,$bgcolor,$title,$url);

	my $calmmdd = substr($caldate,4,4);

	# *************************************************************** #
	# *  Loop thru each event in the data file and get the ones	* #
	# *  that need to be printed for this date. Build an array	* #
	# *  for later printing.					* #
	# *************************************************************** #
	$cellcolor = '';
	$datecolor = '';

	foreach (@caldata) {
		($ndx,$date,$txcolor,$bgcolor,$title,$url,$msg) = split(/\Q$sep/);

		$yyyy	= substr($date,0,4);
		$mmdd	= substr($date,4,4);
		$eventstr = '';

		if ( ($date eq $caldate) || ( ($mmdd eq $calmmdd) && $yyyy < 1) ) {
			$txcolor	= "color=\"$txcolor\"" unless ($txcolor eq '');
			$cellcolor	= "bgcolor=\"$bgcolor\"" unless ($bgcolor eq '');

			if ($msg ne '') {
				$url		= "javascript:msgAlert(${ndx})";
				$msgs[$ndx]	= $title . '\n' . $msg;
				$msgs[$ndx]	=~ s/<br>/\\n/g;
				$msgs[$ndx]	=~ s/'/\\'/g;
			}

			#elsif ($url ne '')      { $link = $url; }
			#else                    { $link = ''; }

			$eventstr	.= "<a href=\"${url}\">" unless ($url eq '');
			$eventstr	.= "<font face=\"Verdana,Comic Sans MS\" $txcolor size=1>";
			$eventstr	.= $title;
			$eventstr	.= '</font>';
			$eventstr	.= '</a>' unless ($url eq '');
			$eventstr	.= '<br>';

			push(@events,$eventstr);
		}
	}

	# *************************************************************** #
	# *  Set up any prettiness needed.				* #
	# *************************************************************** #
	if ($cellcolor ne '') {
		$fontstart	= "<font $txcolor>";
		$fontend	= '</font>';
	} elsif ($caldate eq $today) {
		$fontstart	= "<font color=\"${todaycolor}\" size=5><blink>";
		$fontend	= '</blink></font>';
	} else {
		$fontstart	= '';
		$fontend	= '';
	}


	# *************************************************************** #
	print "<td height=60 $cellcolor width=${colwidth}>";
	print "<b>${fontstart}${i}${fontend}</b><br>";
	foreach (@events) { print; }
	print "</td>\n";
}













# *********************************************************************** #
# *  Print the top of the small calendar table.				* #
# *********************************************************************** #
sub printSmallTableTop {
	print "Content-type: text/html\n\n";
	
	$jsmsgcnt = 0;

	my $curr_month = uc $month[$mon];
	print qq~
		<table border="${small_table_border}" cellpadding="${small_cellpadding}" cellspacing="${small_cellspacing}" bgcolor="${small_table_bgcolor}" width="${small_table_width}">
			<tr>
			  <td class="smcSmallHead" bgcolor="${small_table_bgcolor}" colspan="8">
				  <table border="0" cellpadding="0" cellspacing="0" width="100%">
				  	<tr>
				  		<td class="smcSmallHead" bgcolor="${small_table_bgcolor}"><b>${small_table_caption}</b></td>
				  		<td class="smcSmallHead" bgcolor="${small_table_bgcolor}" align="right"><img src="/images/sitedesign/bars.gif"></td>
				  	</tr>
				  </table>
			  </td>
			</tr>
			<tr>
			  <td class="smcSmallCell" bgcolor="${small_cellcolor}" colspan="8">
				  <table border="0" cellpadding="0" cellspacing="0" width="100%">
				  	<tr>
				  		<td class="smcSmallCell" bgcolor="${small_cellcolor}" align="center"><b>$curr_month -$yy</b></td>
				  	</tr>
				  </table>
			  </td>
			</tr>
	~; 
#				  		<td class="smcSmallHead" bgcolor="${small_table_bgcolor}"><img src="/images/sitedesign/bars.gif"></td>
}
	


sub printSmallTableBottom
{
	print qq~</table>~;
	
	
	return unless ($jsmsgcnt);

	print	"<script type=\"text/javascript\" language=\"javascript\">\n",
		"<!--\n",
		"\tfunction msgAlert (i) {\n",
		"\t\talert('$caltitlejs\\n\\n' + msg[i]);\n",
		"\t}\n",
		"\tmsg = new Array();\n";

	for ($i = 0; $i <= $#msgs; $i++) {
		if ($msgs[$i] ne '') {
			$msgs[$i] =~ s/<br>/\\n/g;
			print	"\tmsg[$i] = '$msgs[$i]';\n";
		}
	}

	print	"// -->\n",
		"</script>\n";
	
}






sub printSmallPrevNext {
	my ($prev,$next) = ("&nbsp;", "&nbsp;");

	if( ($prevmm ne $small_lowest_month) || ($prevyyyy ne $small_lowest_year) )
	{
		# *************************************************************** #
		$prev	= "<a href=\"${thisurl}?o=small&c=$config&mm=$prevmm&yyyy=$prevyyyy\" "
			. "onMouseOver=\"window.status='$month[$prevmm - 1] $prevyyyy';return true\" "
			. "onMouseOut=\"window.status=''\">"
			. "&lt;&lt; $months[$prevmm - 1]</a>";
	}

	if( ($nextmm ne $small_highest_month) || ($nextyyyy ne $small_highest_year) )
	{
		$next	= "<a href=\"${thisurl}?o=small&c=$config&mm=$nextmm&yyyy=$nextyyyy\" "
			. "onMouseOver=\"window.status='$month[$nextmm - 1] $nextyyyy';return true\" "
			. "onMouseOut=\"window.status=''\">"
			. "$months[$nextmm - 1] &gt;&gt;</a>";
	}

	print qq~
	<tr class="smcSmallCell" bgcolor="${small_cellcolor}" align="center">
		<td colspan=\"7\">
		<TABLE CELLPADDING="0" CELLSPACING="0" BORDER="0" WIDTH="100%">
			<tr class="smcSmallCell" bgcolor="${small_cellcolor}" align="center">
			  <TD width="50%" align="left">$prev</TD>
			  <TD width="50%" align="right">$next</td>
			</tr>
		</table>
		</td>
	</tr>
	~;
	
}



sub printSmallWeekDays
{	
	# *************************************************************** #
	print "<tr class=\"smcSmallCell\" bgcolor=\"${small_cellcolor}\" align=\"center\">\n",
		"<th width=${colwidth}>$small_weekdays[(0+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(1+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(2+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(3+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(4+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(5+$small_dayoffset)%7]</th>\n",
		"<th width=${colwidth}>$small_weekdays[(6+$small_dayoffset)%7]</th>\n",
		"</tr>\n";
}




# *********************************************************************** #
# *  Print the main calendar data.					* #
# *********************************************************************** #
sub printSmallCalendar {

	$colspanvar	= (7-$small_dayoffset+$wday)%7;
	$begindates	= 0;
	$daynumber	= 0;

	my $table_row = "<tr class=\"smcSmallCell\" bgcolor=\"${small_cellcolor}\" align=\"center\">\n";
	
	print $table_row;
			
	# *************************************************************** #
	# *  Start the dates loop.					* #
	# *************************************************************** #
	DAY:
	for ($i = 1; $i <= $mday; $i++) {
		$daynumber++;

		# ******************************************************* #
		# *  Handle the beginning of the month.			* #
		# ******************************************************* #
		if ( !($begindates) && ($colspanvar > 0) ) {
			for(my $prefill=0; $prefill<$colspanvar; $prefill++)
			{
				print "<td>&nbsp;</td>\n";
			}
			$begindates = 1;
			$daynumber = $colspanvar;
			$i = 0;
			next DAY;
		}

		# ******************************************************* #
		# *  Generate the date string that will be sent to the	* #
		# *  printEvents subroutine.				* #
		# ******************************************************* #
		$dd = ($i < 10) ? '0' . $i : $i;
		$caldate = $yyyy . $mm . $dd;

		# ******************************************************* #
		# *  Print the HTML code for this date.			* #
		# ******************************************************* #
		&printSmallEvents($caldate);

		# ******************************************************* #
		# *  Handle then end of week.				* #
		# ******************************************************* #
		$mod = ($daynumber % 7);
		if ( ($mod == 0) && ($i < $mday) ) {
			#print "</tr>\n<!--[$daynumber][$i][$dd]-->\n<tr valign=top>\n";
			print "</tr>\n$table_row";
		}
	}

	# *************************************************************** #
	# *  Handle then end of month.					* #
	# *************************************************************** #
	if ( $mod != 0 ) {
		$colspanvar = ( 7 - $mod);
		for(my $remain=0; $remain<$colspanvar; $remain++)
		{
			print "<td>&nbsp;</td>\n";
		}
	}

	print	"</tr>\n\n";

}




# *********************************************************************** #
# *  Subroutine to find and print calendar data. The date passed is in	* #
# *  the form 'yyyymmdd'.						* #
# *********************************************************************** #
sub printSmallEvents {
	local($caldate) = @_;
	local($date,$size,$category,$color,$string);
	local($yyyy,$mmdd);

	my($date,$txcolor,$bgcolor,$title,$url);

	my $calmmdd = substr($caldate,4,4);

	# *************************************************************** #
	# *  Loop thru each event in the data file and get the ones	* #
	# *  that need to be printed for this date. Build an array	* #
	# *  for later printing.					* #
	# *************************************************************** #
	$cellcolor = '';
	$eventstr  = '';
    $celltitle = '';
    $fontstart	= '';
    $fontend	= '';
    $cellurl	= '';

	foreach (@caldata) {
		($ndx,$date,$txcolor,$bgcolor,$title,$url,$msg) = split(/\Q$sep/);

		$yyyy	= substr($date,0,4);
		$mmdd	= substr($date,4,4);
#		$eventstr = "${i}";

		if ( ($date eq $caldate) || ( ($mmdd eq $calmmdd) && $yyyy < 1) ) {
			$cellcolor	= " bgcolor=\"$bgcolor\"" unless ($bgcolor eq '');

			if ($msg ne '') {
				$cellurl		= "javascript:msgAlert(${ndx})";
				$msgs[$ndx]	= $title . '\n' . $msg;
				$msgs[$ndx]	=~ s/<br>/\\n/g;
				$msgs[$ndx]	=~ s/'/\\'/g;

				$jsmsgcnt++;
			}
            elsif( ($url ne '') && $small_addquery)
            {
                $cellurl = "${url}?${query_string}";
            }
            else
            {
                $cellurl = $url;
            }

            $celltitle = " title=\"$title\"" unless($title eq '');
      		$fontstart	= "<font color=\"${txcolor}\">" unless($txcolor eq '');
		    $fontend	= '</font>' unless ($txcolor eq '');

#            if($url eq '')
#            {
#   			$eventstr	= "${fontstart}${i}${fontend}";
#            }
#            else
#            {
#    			$eventstr	= "<a href=\"${url}\">${fontstart}${i}${fontend}</a>";
#            }

            last;
		}
	}

    if ($caldate eq $today)
    {
        $cellcolor  = " bgcolor=\"${small_today_cellcolor}\"";
        $fontstart	= "<font color=\"${small_today_txcolor}\">";
	    $fontend	= '</font>';
    }

 	# *************************************************************** #
	print "<td${cellcolor}${celltitle}>";
	print "<A HREF=\"${cellurl}\">" unless($cellurl eq '');
	print "<b>${fontstart}${i}${fontend}</b>";
	print "</A>" unless($cellurl eq '');
	print "</td>\n";

}
























# *********************************************************************** #
# *  Subroutine to get the month and year from the query string.	* #
# *********************************************************************** #
sub getOptions {
	local(*FORM_DATA) = @_;
	my(@key_value_pairs,$key_value, $key, $value, $url);

	($url, $query_string) = split(/\?/, $ENV{'REQUEST_URI'});
	$query_string = $ENV{'QUERY_STRING'} unless ($query_string);

	@key_value_pairs = split (/&/, $query_string);

	foreach $key_value (@key_value_pairs) {
		($key, $value) = split (/=/, $key_value);
		$value =~ tr/+/ /;
		$value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack ("C", hex ($1))/eg;
		$FORM_DATA{$key} = $value;
        }
}







# *********************************************************************** #
# *  Subroutine to get the day of week of the date passed in.		* #
# *********************************************************************** #
sub dayofweek {
	local($y) = $_[0];
	local($m) = $_[1];
	local($d) = $_[2];
	local(@d) = (0,3,2,5,0,3,5,1,4,6,2,4);
	$y-- if $m < 3;
	(($y+int($y/4)-int($y/100)+int($y/400)+$d[$m-1]+$d) % 7);
}


# *********************************************************************** #
# *  This subroutine accepts the date and a number of days in an array. * #
# *  It adds that number of days to the date and returns the result.    * #
# *********************************************************************** #
sub dateadd {
	my ($yyyy,$mm,$dd,$add) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
	my ($seconds_to_add,$idate);

	return ('','','') if (($yyyy eq '') || ($mm eq '') || ($dd eq ''));

	$seconds_to_add = ($add * 86400);

	$mday	= $dd;
	$mon	= ($mm - 1);
	$year	= ($yyyy - 1900);

	$sec	= $min = $hour = 4;
	$wday	= $yday = $isdst = '';

	$idate	= (&timelocal($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) + $seconds_to_add);
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($idate);

	$dd	= sprintf("%02d", $mday);
	$mm	= sprintf("%02d", $mon + 1);
	$yyyy	= ($year + 1900);

	return($yyyy . $mm . $dd);
}


# *********************************************************************** #
# *  Subroutine to get the month and year from the query string.	* #
# *********************************************************************** #
sub gracefulError {
	my($msg) = @_;
	print	"Content-type: text/html\n\n";

	print	"<html>\n",
		"<head>\n",
		"<title>ERROR</title>\n",
		"</head>\n",
		"<body bgcolor=#ffffff text=#000000><tt>\n\n",
		"<font color=red size=6><b>ERROR</b></font>\n<p>\n";

	print	$msg;

	print	"\n</tt></body>\n",
		"</html>\n";

	exit(1);
}

# *********************************************************************** #
# *				Done.					* #
# *********************************************************************** #
